﻿//
//   (c) Copyright Agilent Technologies 2012
//              All rights reserved.
//
//  Customer shall have the personal, non-
// transferable rights to use, copy or modify
// this SAMPLE PROGRAM for Customer's internal
// operations. Customer shall use the SAMPLE
// PROGRAM solely and exclusively for its own
// purpose and shall not license, lease, market
// or distribute the SAMPLE PROGRAM or modification
// or any part thereof.
//
//  Agilent shall not be liable for the quality,
// performance or behavior of the SAMPLE PROGRAM.
// Agilent especially disclaims that the operation
// of the SAMPLE PROGRAM shall be uninterrupted or
// error free. This SAMPLE PROGRAM is provided
// AS IS.
//
//  AGILENT DISCLAIMS THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE.
//
//  Agilent shall not be liable for any infringement
// of any patent, trademark, copyright or other
// proprietary rights by the SAMPLE PROGRAM or
// its use. Agilent does not warrant that the SAMPLE
// PROGRAM is free from infringements or such
// rights of third parties. However, Agilent will not
// knowingly infringe or deliver a software that
// infringes the patent, trademark, copyright or
// other proprietary right of a third party.
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace B2900CsSample
{
    public partial class MainWindow : Form
    {
        private SmuControl smu = new SmuControl();
        private DnvmControl dnvm = new DnvmControl();


        public MainWindow()
        {
            InitializeComponent();
        }

        private void MainWindow_Load(object sender, EventArgs e)
        {
            // Loading help document from "HelpDocument" resource
            try
            {
                System.Reflection.Assembly asm = this.GetType().Assembly;
                System.Resources.ResourceManager rm = new System.Resources.ResourceManager("B2900CsSample.Properties.Resources", asm);
                string rtfstr = (string)rm.GetObject("HelpDocument");
                richTextBoxHelp.Rtf = rtfstr;
            }
            catch (Exception ex)
            {
                richTextBoxHelp.WordWrap = true;
                richTextBoxHelp.Text = "Cannot find help document." + Environment.NewLine + Environment.NewLine;
                richTextBoxHelp.Text += ex.Message; 
            }
        }

        private void buttonRun_Click(object sender, EventArgs e)
        {
            string address1;
            string address2;
            string sweepMethod ="";
            double ch1startC, ch1stopC, ch1compV;
            double intgPlc, measPlc;

            int nofstep;
            int sweepType = -1;
            bool ret;


            // Get parameters & Check parameter syntax
            address1 = comboBoxVisaResource1.Text.Trim();
            address2 = comboBoxVisaResource2.Text.Trim();

            if ( address1 == "" )
            {
                MessageBox.Show(this, "B2960A Resource String is empty.", "Error");
                return;
            }
            if (address2 == "")
            {
                MessageBox.Show(this, "34420A Resource String is empty.", "Error");
                return;
            }
            if (!double.TryParse(textBoxCh1StartC.Text, out ch1startC))
            {
                MessageBox.Show(this, "Syntax error in start current.", "Error");
                return;
            }
            if (!double.TryParse(textBoxCh1StopC.Text, out ch1stopC))
            {
                MessageBox.Show(this, "Syntax error in stop current.", "Error");
                return;
            }
            if (!double.TryParse(textBoxCh1VLimit.Text, out ch1compV))
            {
                MessageBox.Show(this, "Syntax error in channel 1 limit(compliance) current.", "Error");
                return;
            }
            if (!double.TryParse(textBoxMeasureSpeed.Text, out measPlc))
            {
                MessageBox.Show(this, "Syntax error in B2960A PLC.", "Error");
                return;
            }
            if (!double.TryParse(comboBoxNPLC.Text, out intgPlc))
            {
                MessageBox.Show(this, "Syntax error in 34420A PLC.", "Error");
                return;
            }

            nofstep = (int)numericUpDownNofStep.Value;

            // Check SweepMethod radioButton
            foreach (RadioButton rbMethod in groupBoxMethod.Controls)
            {
                if (rbMethod.Checked)
                {
                    sweepMethod = rbMethod.Text;
                    break;
                }
            }

            try
            {
                // Disable buttons
                buttonRun.Enabled = false;
                buttonSave.Enabled = false;

                //// Set & Measure
                // Open B2960A
                ret = smu.OpenDevice(address1);
                textBoxModel1.Text = smu.ModelName;
                if (!ret)
                {
                    MessageBox.Show(this, "Failed to open a device. [B2960A]" + Environment.NewLine + smu.ErrorMessage, "Error");
                    return;
                }

                // Open 34420A
                if (address2 != "None")
                {
                    ret = dnvm.OpenDevice(address2);
                    textBoxModel2.Text = dnvm.ModelName;
                    if (!ret)
                    {
                        MessageBox.Show(this, "Failed to open a device. [34420A]" + Environment.NewLine + dnvm.ErrorMessage, "Error");
                        return;
                    }
                }
                Application.DoEvents();

                // Set Sweep Method Code.
                switch (sweepMethod)
                {
                    case "Normal":          sweepType = 1;      break;

                    case "NULL":            sweepType = 2;      break;

                    case "Forward/Reverse": sweepType = 3;      break;

                    default:                sweepType = -1;     break;
                }

                // Set Sweep Channel parameter
                if (!smu.SetSweepChannel(1, ch1startC, ch1stopC, nofstep, ch1compV, measPlc, sweepType))
                {
                    MessageBox.Show(this, "Failed to set sweep condition." + Environment.NewLine + smu.ErrorMessage, "Error");
                    return;
                }


                // Start Measurement
                if (address2 != "None")
                {
                    if (!dnvm.Initiate(nofstep * sweepType, intgPlc))      // Initiate 34420A
                    {
                        MessageBox.Show(this, "Failed to start measurement. [34420A]" + Environment.NewLine + dnvm.ErrorMessage, "Error");
                        return;
                    }
                    System.Threading.Thread.Sleep(1000);                        // Wait 1sec.
                }

                if (!smu.Initiate(new int[] {1}))                               // Initiate B2960A
                {
                    MessageBox.Show(this, "Failed to start measurement. [B2960A]" + Environment.NewLine + smu.ErrorMessage, "Error");
                    return;
                }


                // Clear Graph & Table
                dataGridView1.Rows.Clear();
                chart1.Series[0].Points.Clear();
                chart1.Series[1].Points.Clear();

                // Read measured data
                double xmax = double.MinValue;
                double xmin = double.MaxValue;
                double ymax = double.MinValue;
                double ymin = double.MaxValue;
                double regValue = 0.0;
                const int READ_SIZE = 24;
                double[,] data = new double[READ_SIZE, SmuControl.COLUMN_COUNT];
                bool isFirst = true;
                int actrow;


                // Get B2960A Measurement Data
                do
                {
                    if (!smu.GetData(1, isFirst, READ_SIZE, ref data, out actrow))
                    {
                        MessageBox.Show(this, "Failed to read measurement result from SMU." + Environment.NewLine + smu.ErrorMessage, "Error");
                        return;
                    }
                    isFirst = false;
                    if (actrow == 0) break;

                    // Update Table and graph
                    for (int i = 0; i < actrow; i++)
                    {
                        // [Sweep method : Fw/Rv] & [Source : 0[A]] -> Skip
                        if (sweepType == 3 && (i % 3) == 0) continue;

                        // Add data to table
                        int row = dataGridView1.Rows.Add(data[i, 1], data[i, 0], "---", data[i, 2], data[i, 3], data[i, 4], "---", "---");
                        dataGridView1.Rows[row].HeaderCell.Value = (row + 1).ToString();

                        // Calculate Resistance
                        regValue = CalcResistance(row, 0, 1, sweepType);

                        if (regValue != 9.91e37)    // Invalid value -> Don't use.
                        {
                            double targetC;

                            // Add resistance to table & Check current value for prot graph
                            if (sweepType == 3){       // Sweep Method : Fw/Rv  -> Target row : Positive source.
                                dataGridView1.Rows[row - 1].Cells[2].Value = regValue;
                                targetC = (double)dataGridView1.Rows[row - 1].Cells[0].Value;
                            }
                            else{
                                dataGridView1.Rows[row].Cells[2].Value = regValue;
                                targetC = (double)dataGridView1.Rows[row].Cells[0].Value;
                            }

                            // Set data to Graph [X-Current, Y-Registance]
                            chart1.Series[0].Points.AddXY(targetC, regValue);

                            // Check min/MAX :  X,Y-Axis
                            if (targetC > xmax) xmax = targetC;
                            if (targetC < xmin) xmin = targetC;
                            if (regValue > ymax) ymax = regValue;
                            if (regValue < ymin) ymin = regValue;
                        }
                    }

                    scaleAdjust(false, chart1.ChartAreas[0], xmin, xmax, ymin, ymax);
                    Application.DoEvents();

                } while (true);


                if (address2 != "None")
                {
                    // Get 34420A Measurement Data
                    double[] data2 = new double[nofstep*sweepType];
                    if (!dnvm.GetData(data2))
                    {
                        MessageBox.Show(this, "Failed to read measurement result from DNVM." + Environment.NewLine + dnvm.ErrorMessage, "Error");
                        return;
                    }

                    int rowCount = 0;

                    for (int i = 0; i < data2.Length; i++)
                    {
                        // [Sweep method : Fw/Rv] & [Source : 0] -> Skip
                        if (sweepType == 3 && (i % 3) == 0) continue;

                        // Add voltage to Table
                        dataGridView1.Rows[rowCount].Cells[6].Value = data2[i];

                        // Calculate Resistance
                        regValue = CalcResistance(rowCount, 0, 6, sweepType);

                        if (regValue != 9.91e37)    // Invalid value -> Don't use.
                        {
                            double targetC;

                            // Add resistance to table & Check current value for prot graph
                            if (sweepType == 3){   // Sweep Method : Fw/Rv     -> Get Positive Value
                                dataGridView1.Rows[rowCount - 1].Cells[7].Value = regValue;
                                targetC = (double)dataGridView1.Rows[rowCount - 1].Cells[0].Value;
                            }
                            else {
                                dataGridView1.Rows[rowCount].Cells[7].Value = regValue;
                                targetC = (double)dataGridView1.Rows[rowCount].Cells[0].Value;
                            }

                            // Set data to Graph
                            chart1.Series[1].Points.AddXY(targetC, regValue);

                            // Check min/MAX : Y-Axis
                            if (regValue > ymax) ymax = regValue;
                            if (regValue < ymin) ymin = regValue;
                        }
                        rowCount++;
                   }

                }

                scaleAdjust(true, chart1.ChartAreas[0], xmin, xmax, ymin - (System.Math.Abs((ymax - ymin) / 20.0)), ymax + (System.Math.Abs((ymax - ymin) / 20.0)));
            }

            catch
            {
            }
            finally
            {
                // Close
                smu.CloseDevice();
                if (address2 != "None")
                {
                    dnvm.CloseDevice();
                }

                // Enable buttons
                buttonRun.Enabled = true;
                buttonSave.Enabled = true;
            }

        }

        private void scaleAdjust(bool isLast, System.Windows.Forms.DataVisualization.Charting.ChartArea chartaria, 
                                 double xDataMin, double xDataMax, double yDataMin, double yDataMax)
        {
            double resolution;
            resolution = Math.Pow(10.0,(int)Math.Floor(Math.Log10(xDataMax - xDataMin)) - 1);
            double new_xAxisMin = Math.Ceiling(xDataMin / resolution) * resolution;
            double new_xAxisMax = Math.Floor(xDataMax / resolution) * resolution;
            resolution = Math.Pow(10.0, (int)Math.Floor(Math.Log10(yDataMax - yDataMin)) - 1);
            double new_yAxisMin = Math.Floor(yDataMin / resolution) * resolution;
            double new_yAxisMax = Math.Ceiling(yDataMax / resolution) * resolution;
            if (isLast)
            {
                chartaria.AxisX.Minimum = new_xAxisMin;
                chartaria.AxisX.Maximum = new_xAxisMax;
                chartaria.AxisY.Minimum = new_yAxisMin;
                chartaria.AxisY.Maximum = new_yAxisMax;
            }
            else
            {
                if (chartaria.AxisX.Minimum > new_xAxisMin) chartaria.AxisX.Minimum = new_xAxisMin;
                if (chartaria.AxisX.Maximum < new_xAxisMax) chartaria.AxisX.Maximum = new_xAxisMax;
                if (chartaria.AxisY.Minimum > new_yAxisMin) chartaria.AxisY.Minimum = new_yAxisMin;
                if (chartaria.AxisY.Maximum < new_yAxisMax) chartaria.AxisY.Maximum = new_yAxisMax;
            }
        }


        /// <summary>
        /// Calculate resistance from table value.
        /// </summary>
        /// <param name="rowPos">Row position</param>
        /// <param name="currentCell">Current cell position</param>
        /// <param name="voltageCell">Voltage cell position</param>
        /// <param name="sweepType">Sweep Method [1-3]</param>
        /// <returns>Resistance value</returns>
        private double CalcResistance(int rowPos, int currentCell, int voltageCell, int sweepType)
        {
            double resistance = 9.91e37;    // Default : Invalid value
            double difC;                    // Current Diff.

            switch (sweepType)
            {
                case 1:         // Sweep Method : Normal
                    if ((double)dataGridView1.Rows[rowPos].Cells[currentCell].Value != 0.0)
                    {
                        resistance = (double)dataGridView1.Rows[rowPos].Cells[voltageCell].Value / (double)dataGridView1.Rows[rowPos].Cells[currentCell].Value; 
                    }
                    break;

                case 2:         // Sweep Method : NULL
                    if ((rowPos%2)!= 0)         // Odd row.
                    {
                        difC = (double)dataGridView1.Rows[rowPos].Cells[currentCell].Value - (double)dataGridView1.Rows[rowPos - 1].Cells[currentCell].Value;
                        if (difC != 0.0)
                        {
                            resistance = ((double)dataGridView1.Rows[rowPos].Cells[voltageCell].Value - (double)dataGridView1.Rows[rowPos - 1].Cells[voltageCell].Value) / difC;
                        }
                    }
                    break;

                case 3:         // Sweep Method : Forward/Reverse
                    if ((rowPos % 2) != 0)     // Odd row.
                    {
                        difC = (double)dataGridView1.Rows[rowPos - 1].Cells[currentCell].Value - (double)dataGridView1.Rows[rowPos].Cells[currentCell].Value;
                        if (difC != 0.0)
                        {
                            resistance = ((double)dataGridView1.Rows[rowPos - 1].Cells[voltageCell].Value - (double)dataGridView1.Rows[rowPos].Cells[voltageCell].Value) / difC;
                        }
                    }

                    break;

                default:
                    break;
            }

            return resistance;
        }




        private void buttonSave_Click(object sender, EventArgs e)
        {
            // Modify filename with current timestamp
            string lastfilename = saveFileDialog1.FileName;
            int parenthesis1 = lastfilename.IndexOf('(');
            int parenthesis2 = lastfilename.LastIndexOf(')');
            if (parenthesis1 >= 0 && parenthesis2 >= 0 && parenthesis2 > parenthesis1)
            {
                saveFileDialog1.FileName = lastfilename.Remove(parenthesis1 + 1, parenthesis2 - parenthesis1 - 1)
                                                       .Insert(parenthesis1 + 1,DateTime.Now.ToString("yyyyMMddHHmmss"));
            }

            // Save CVS file
            if (saveFileDialog1.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    // Open file
                    System.IO.StreamWriter tw = new System.IO.StreamWriter(saveFileDialog1.FileName);

                    // Write header
                    int row, col;
                    for (col = 0; col < dataGridView1.Columns.Count; col++)
                    {
                        if (col == 0) tw.Write(string.Format("{0,-8:s}",dataGridView1.Columns[col].HeaderText));
                        else tw.Write("," + string.Format("{0,-8:s}", dataGridView1.Columns[col].HeaderText));
                    }
                    tw.Write(Environment.NewLine);

                    // Write table data
                    for (row = 0; row < dataGridView1.Rows.Count; row++)
                    {
                        for (col = 0; col < dataGridView1.Columns.Count; col++)
                        {
                            if (col == 0) tw.Write(string.Format("{0,-8:s}",dataGridView1.Rows[row].Cells[col].Value.ToString()));
                            else tw.Write("," + string.Format("{0,-8:s}",dataGridView1.Rows[row].Cells[col].Value.ToString()));
                        }
                        tw.Write(Environment.NewLine);
                    }

                    // Close file
                    tw.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(this, "Failed to save CSV file." + Environment.NewLine + ex.Message, "Error");
                }
            }
        }
    }
}
